/*
 *  PHEX - The pure-java Gnutella-servent.
 *  Copyright (C) 2001 - 2006 Phex Development Group
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  Created on 05.01.2006
 *  --- CVS Information ---
 *  $Id: PingWorker.java 3638 2006-12-01 10:14:16Z GregorK $
 */
package phex.connection;

import phex.common.ThreadTracking;
import phex.host.HostManager;
import phex.msg.MsgManager;

/**
 * This class is responsible to ping the neighborhod and find new hosts for the
 * PongCache and for the CaughtHostsContainer.
 */
public class PingWorker implements Runnable
{
    private static final int SLEEP_TIME = 5000;
    private MsgManager msgMgr;
    private HostManager hostMgr;
    
    public PingWorker()
    {
        hostMgr = HostManager.getInstance();
        msgMgr = MsgManager.getInstance();
    }
    
    public void start()
    {
        Thread thread = new Thread( ThreadTracking.rootThreadGroup, this,
            "PingWorker-" + Integer.toHexString( hashCode() ) );
        thread.setPriority( Thread.NORM_PRIORITY );
        thread.setDaemon( true );
        thread.start();
    }
    
    public void run()
    {
        while( true )
        {
            // Sleep some time...
            try
            {
                Thread.sleep( SLEEP_TIME );
            }
            catch (InterruptedException e)
            {
            }
            
            if( hostMgr.isUltrapeer() )
            {
                msgMgr.broadcastPingHosts( (byte)3 );
            }
        }
    }
}